//****************************************************************************
// CLASS NAME:	NewDesignDialog.java
//
// AUTHOR:	Rick Brink
//		    rick@mail.csh.rit.edu
//		    http://www.csh.rit.edu/~rick
//
// VERSION:	1.0
//
// HISTORY:	4/16/96		Initial Version
//
// COPYRIGHT INFORMATION:
//
// This program and the Java source is in the public domain.
// Permission to use, copy, modify, and distribute this software
// and its documentation for NON-COMMERCIAL purposes and
// without fee is hereby granted.
//
//    Copyright 1996
//
//    Rick Brink
//    1266 Brighton-Henrietta Townline Rd.
//    Rochester, NY 14623
//
// DISCLAIMER:
//
// The author claims no responsibility for any damage, direct or indirect,
// to any harware or software as a result of using this program.
//****************************************************************************

package PetriTool;

/**
  * A class representing a Dialog which asks the user if they
  * want to start a new design, and presents them with two
  * buttons (yes/no).
  *
  * @see YesNoDialog
  * @see PetriToolFrame
  *
  * @version 1.0 July 3, 1996
  *
  * @author  Rick Brink
**/
public class NewDesignDialog extends YesNoDialog {
    /**
      * Handle to our caller so we can take certain actions
      * based on the user's response.
    **/
    protected PetriToolFrame parent_;

    /**
      * Create the kind of YesNoDialog we want with our question
      * and two buttons (yes and no).
      *
      * @param parent PetriToolFrame that called this object
    **/
    public NewDesignDialog(PetriToolFrame parent) {
        super(parent, "New Design", "Are you sure you want to clear " +
        	"the current design?", "Yes", "No", null);
        parent_ = parent;

    }

    /**
      * Action to take if user responds with the Yes button.
      * Set the default parameters for PetriTool, clear the
      * filename and directory we were using for the last design,
      * and create a new DesignPanel.
    **/
    public void yes() {
        parent_.petriTool_.newDesign();
        parent_.petriTool_.designPanel_.newDesign();
        parent_.newDesign();
    }

    /**
      * Action to take if user responds with the No button.
      * Simply write a message confirming the cancellation of
      * the new design.
    **/
    public void no() {
        parent_.StatusMessage("New design canceled.");
    }
}
